@@ -15,7 +15,7 @@ except ImportError:
 
 from valid8 import ValidationFailure, is_pep484_nonable
 
-from pyfields.typing_utils import assert_is_of_type, FieldTypeError
+from pyfields.typing_utils import assert_is_of_type, FieldTypeError, get_type_hints
 from pyfields.validate_n_convert import FieldValidator, make_converters_list, trace_convert
 
 try:  # python 3.5+
@@ -37,13 +37,6 @@ USE_ADVANCED_TYPE_CHECKER = assert_is_of_type is not None
 
 
 PY36 = sys.version_info >= (3, 6)
-get_type_hints = None
-if PY36:
-    try:
-        from typing import get_type_hints
-    except ImportError:
-        pass
-
 PY2 = sys.version_info < (3, 0)
 # PY35 = sys.version_info >= (3, 5)
 
@@ -113,6 +106,7 @@ class Symbols(Enum):
     EMPTY = 2  # type: Any
     USE_FACTORY = 3
     _unset = 4
+    DELAYED = 5
 
     def __repr__(self):
         """ More compact representation for signatures readability"""
@@ -127,6 +121,7 @@ UNKNOWN = Symbols.UNKNOWN
 
 # EMPTY = sentinel.create('empty')
 EMPTY = Symbols.EMPTY
+DELAYED = Symbols.DELAYED
 
 # USE_FACTORY = sentinel.create('use_factory')
 USE_FACTORY = Symbols.USE_FACTORY
@@ -258,16 +253,19 @@ class Field(object):
             self.name = name
 
         # if not already manually overridden, get the type hints if there are some in the owner class annotations
-        if self.type_hint is EMPTY:
+        if self.type_hint is EMPTY or self.type_hint is DELAYED:
+            # first reconciliate both ways to get the hint
             if owner_cls_type_hints is not None:
                 if type_hint is not None:
                     raise ValueError("Provide either owner_cls_type_hints or type_hint, not both")
                 type_hint = owner_cls_type_hints.get(name)
 
+            # then use it
             if type_hint is not None:
                 # only use type hint if not empty
                 self.type_hint = type_hint
-                # update the 'nonable' status
+                # update the 'nonable' status - only if not already explicitly set.
+                # note: if this is UNKNOWN, we already saw that self.default is not None. No need to check again.
                 if self.nonable is UNKNOWN:
                     if is_pep484_nonable(type_hint):
                         self.nonable = True
@@ -296,8 +294,17 @@ class Field(object):
                      ):
         if owner is not None:
             # fill all the information about how it is attached to the class
-            cls_type_hints = get_type_hints(owner)
-            self.set_as_cls_member(owner, name, owner_cls_type_hints=cls_type_hints)
+            # resolve type hint strings and get "optional" type hint automatically
+            # note: we need to pass an appropriate local namespace so that forward refs work.
+            # this seems like a bug in `get_type_hints` ?
+            try:
+                cls_type_hints = get_type_hints(owner)
+            except NameError:
+                # probably an issue of forward reference, or PEP563 is activated. Delay checking for later
+                self.set_as_cls_member(owner, name, type_hint=DELAYED)
+            else:
+                # nominal usage
+                self.set_as_cls_member(owner, name, owner_cls_type_hints=cls_type_hints)
 
     @property
     def qualname(self):
@@ -809,18 +816,13 @@ class NativeField(Field):
 
         # do this first, because a field might be referenced from its class the first time it will be used
         # for example if in `make_init` we use a field defined in another class, that was not yet accessed on instance.
-        if not PY36 and self.name is None:
+        if self.name is None or self.type_hint is DELAYED:
             # __set_name__ was not called yet. lazy-fix the name and type hints
             fix_field(obj_type, self)
 
         if obj is None:
-            # class-level call ?
-            # TODO put back when https://youtrack.jetbrains.com/issue/PY-38151 is solved
-            # return self
-            # even this does not work
-            # exec("o = self", globals(), locals())
-            # return locals()['o']
-            raise ClassFieldAccessError(self)
+            # class-level call: https://youtrack.jetbrains.com/issue/PY-38151 is solved, we can now return self
+            return self
 
         # Check if the field is already set in the object __dict__
         value = obj.__dict__.get(self.name, _unset)
@@ -998,15 +1000,13 @@ class DescriptorField(Field):
 
         # do this first, because a field might be referenced from its class the first time it will be used
         # for example if in `make_init` we use a field defined in another class, that was not yet accessed on instance.
-        if not PY36 and self.name is None:
+        if self.name is None or self.type_hint is DELAYED:
             # __set_name__ was not called yet. lazy-fix the name and type hints
             fix_field(obj_type, self)
 
         if obj is None:
-            # class-level call ?
-            # TODO put back when https://youtrack.jetbrains.com/issue/PY-38151 is solved
-            # return self
-            raise ClassFieldAccessError(self)
+            # class-level call: https://youtrack.jetbrains.com/issue/PY-38151 is solved, we can now return self
+            return self
 
         private_name = '_' + self.name
 
@@ -1066,15 +1066,14 @@ class DescriptorField(Field):
 
         # do this first, because a field might be referenced from its class the first time it will be used
         # for example if in `make_init` we use a field defined in another class, that was not yet accessed on instance.
-        if not PY36 and self.name is None:
+        if self.name is None or self.type_hint is DELAYED:
             # __set_name__ was not called yet. lazy-fix the name and type hints
             fix_field(obj.__class__, self)
 
-        if obj is None:
-            # class-level call ?
-            # TODO put back when https://youtrack.jetbrains.com/issue/PY-38151 is solved
-            # return self
-            raise ClassFieldAccessError(self)
+        # if obj is None:
+        #     # class-level call: this never happens
+        #     # https://youtrack.jetbrains.com/issue/PY-38151 is solved, but what do we wish to do here actually ?
+        #     raise ClassFieldAccessError(self)
 
         if self.converters is not None:
             # this is an inlined version of `trace_convert` with no capture of details
@@ -1118,10 +1117,10 @@ class DescriptorField(Field):
             # check the type
             if self.check_type:
                 if t is EMPTY:
-                    raise ValueError("`check_type` is enabled on field '%s' but no type hint is available. Please provide"
-                                     "type hints or set `field.check_type` to `False`. Note that python code is not able to"
-                                     " read type comments so if you wish to be compliant with python < 3.6 you'll have to"
-                                     "set the type hint explicitly in `field.type_hint` instead")
+                    raise ValueError("`check_type` is enabled on field '%s' but no type hint is available. Please "
+                                     "provide type hints or set `field.check_type` to `False`. Note that python code is"
+                                     " not able to read type comments so if you wish to be compliant with python < 3.6 "
+                                     "you'll have to set the type hint explicitly in `field.type_hint` instead")
 
                 if USE_ADVANCED_TYPE_CHECKER:
                     # take into account all the subtleties from `typing` module by relying on 3d party providers.
